@@ -434,9 +434,9 @@ static void
 hildon_file_chooser_dialog_do_autonaming(HildonFileChooserDialogPrivate *
                                          priv)
 {
+    gboolean changed;
+
     g_assert(HILDON_IS_FILE_SELECTION(priv->filetree));
-	
-	gboolean changed;
 
     if (GTK_WIDGET_VISIBLE(priv->caption_control_name) &&
         priv->stub_name && priv->stub_name[0] && !priv->edited)
@@ -1204,13 +1204,23 @@ static GtkWidget
                                                   action)
 {
     GtkWidget *dialog;
+    gboolean local_only = FALSE;
+    gboolean show_hidden = FALSE;
+    gboolean show_upnp = TRUE;
+    
+    g_object_get (self->priv->filetree, "local-only", &local_only, NULL);
+    g_object_get (self->priv->filetree, "show-hidden", &show_hidden, NULL);
+    g_object_get (self->priv->filetree, "show-upnp", &show_upnp, NULL);
 
-    dialog =
-        hildon_file_chooser_dialog_new_with_properties(GTK_WINDOW(self),
-                                                       "action", action,
-                                                       "file-system-model",
-                                                       self->priv->model,
-                                                       NULL);
+    dialog = hildon_file_chooser_dialog_new_with_properties
+      (GTK_WINDOW(self),
+       "action", action,
+       "file-system-model",
+       self->priv->model, 
+       "local-only", local_only,
+       "show-hidden", show_hidden,
+       "show-upnp", show_upnp,
+       NULL);
     gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
     sync_current_folders(self, HILDON_FILE_CHOOSER_DIALOG(dialog));
 
@@ -1227,7 +1237,11 @@ static void create_folder_callback(GtkFileSystemHandle *handle,
     g_assert(HILDON_IS_FILE_CHOOSER_DIALOG(data));
     self = HILDON_FILE_CHOOSER_DIALOG(data);
 
-    g_assert(self->priv->create_folder_handle == handle);
+    /* There can be still pending cancelled handles
+     * from previous operations, just ignore them
+     */
+    if (self->priv->create_folder_handle != handle)
+	    return;
 
     self->priv->create_folder_handle = NULL;
     dialog = GTK_DIALOG(self);
@@ -1257,6 +1271,7 @@ static void handle_folder_popup(HildonFileChooserDialog *self)
   GtkWidget *dialog;
   gint response;
   gchar *uri = NULL;
+  gboolean edit_entry = FALSE;
 
   g_return_if_fail(HILDON_IS_FILE_CHOOSER_DIALOG(self));
 
@@ -1271,14 +1286,6 @@ static void handle_folder_popup(HildonFileChooserDialog *self)
 
   self->priv->popup_protect = TRUE;
 
-  if (self->priv->edited)
-  {
-    self->priv->edited = FALSE;
-    hildon_file_chooser_dialog_set_current_name 
-      (GTK_FILE_CHOOSER(self),
-       get_entry (self->priv->entry_name, self->priv->ext_name));
-  }
-
   backend = _hildon_file_system_model_get_file_system(self->priv->model);
 
   if (self->priv->action == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER)
@@ -1334,6 +1341,7 @@ static void handle_folder_popup(HildonFileChooserDialog *self)
 			hildon_file_selection_move_cursor_to_uri (self->priv->filetree, 
 								  uri);
 		}
+	edit_entry = TRUE;
       }
     gtk_file_path_free(file_path);
   }
@@ -1343,7 +1351,18 @@ static void handle_folder_popup(HildonFileChooserDialog *self)
                GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
 
     if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
-      sync_current_folders(HILDON_FILE_CHOOSER_DIALOG(dialog), self);
+      {
+        sync_current_folders(HILDON_FILE_CHOOSER_DIALOG(dialog), self);
+	edit_entry = TRUE;
+      }
+  }
+
+  if (edit_entry && self->priv->edited)
+  {
+    self->priv->edited = FALSE;
+    hildon_file_chooser_dialog_set_current_name 
+      (GTK_FILE_CHOOSER(self),
+       get_entry (self->priv->entry_name, self->priv->ext_name));
   }
 
   gtk_widget_destroy(dialog);
@@ -1938,6 +1957,10 @@ hildon_file_chooser_entry_changed( GtkWidget * widget,
 
     priv = HILDON_FILE_CHOOSER_DIALOG( data )->priv;
     priv->edited = TRUE;
+
+    g_free (priv->stub_name);
+    priv->stub_name = g_strdup (gtk_entry_get_text (GTK_ENTRY (widget)));
+
     gtk_dialog_set_response_sensitive(GTK_DIALOG(data),
                                           GTK_RESPONSE_OK,
                                           strlen(gtk_entry_get_text(GTK_ENTRY(widget))) > 0);
@@ -2251,6 +2274,20 @@ sync_extensions_combo (HildonFileChooserDialogPrivate *priv)
 
       model = gtk_combo_box_get_model (GTK_COMBO_BOX (priv->extensions_combo));
 
+      /* if there is a valid extension selected, just leave it */
+      if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (priv->extensions_combo), &iter))
+        {
+	  gboolean matches;
+	  gchar *ext;
+
+	  gtk_tree_model_get (model, &iter, 1, &ext, -1);
+	  matches = (strcmp (ext, priv->ext_name + 1) == 0);
+	  g_free (ext);
+
+	  if (matches)
+	    return;
+	}
+
       i = 0;
       valid = gtk_tree_model_get_iter_first (model, &iter);
       while (valid)
