@@ -69,6 +69,9 @@ struct _HildonFileDetailsDialogPrivate {
     GtkWidget *file_location_image, *file_device_image;
     GtkWidget *file_image;
     GtkWidget *ok_button;
+    GtkWidget *scroll;
+
+    GnomeVFSResult change_permissions_result;
 
     GtkTreeRowReference *active_file;
     gboolean checkbox_original_state;
@@ -97,6 +100,8 @@ hildon_file_details_dialog_get_property( GObject *object, guint param_id,
 static void
 hildon_file_details_dialog_response(GtkDialog *dialog, gint response_id);
 
+static void do_line_wrapping (GtkWidget *w, gpointer data);
+
 static GtkDialogClass *file_details_dialog_parent_class = NULL;
 
 GType hildon_file_details_dialog_get_type(void)
@@ -144,6 +149,27 @@ static gboolean write_access(const gchar *uri)
   return result;
 }
 
+static GnomeVFSResult guess_change_permissions_result (const gchar *uri)
+{
+  GnomeVFSFileInfo *info;
+  gboolean result;
+
+  /* Get information about file */
+  info = gnome_vfs_file_info_new ();
+  result = gnome_vfs_get_file_info (uri, info, GNOME_VFS_FILE_INFO_DEFAULT);
+  if (result == GNOME_VFS_OK)
+    {
+      if (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS)
+	result = gnome_vfs_set_file_info (uri, info,
+					  GNOME_VFS_SET_FILE_INFO_PERMISSIONS);
+      else
+	result = GNOME_VFS_ERROR_GENERIC;
+    }
+
+  gnome_vfs_file_info_unref(info);
+  return result;
+}
+
 /* When model deletes a file, we check if our reference is still valid.
    If not, we emit response, which usually closes the dialog */
 static void check_validity(GtkTreeModel *model,
@@ -194,9 +220,9 @@ static void change_state(HildonFileDetailsDialog *self, gboolean readonly)
     }
 
     if (result != GNOME_VFS_OK)
-      hildon_banner_show_information (GTK_WIDGET (self), NULL,
-                                      gnome_vfs_result_to_string (result));
-
+      hildon_banner_show_information (NULL, NULL,
+				      gnome_vfs_result_to_string (result));
+    
     gnome_vfs_file_info_unref(info);
     g_free(uri);
   }
@@ -206,7 +232,7 @@ static void change_state(HildonFileDetailsDialog *self, gboolean readonly)
 static void
 hildon_file_details_dialog_response(GtkDialog *dialog, gint response_id)
 {
-    if (response_id == GTK_RESPONSE_CANCEL)
+    if (response_id == GTK_RESPONSE_OK)
     {
       HildonFileDetailsDialog *self;
       gboolean state;
@@ -215,17 +241,10 @@ hildon_file_details_dialog_response(GtkDialog *dialog, gint response_id)
       state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(self->priv->file_readonly));
 
       if (state != self->priv->checkbox_original_state)
-        change_state(self, self->priv->checkbox_original_state);
+        change_state(self, state);
     }
 }
 
-static void
-hildon_file_details_dialog_read_only_toggled(GtkWidget *widget, gpointer data)
-{
-  change_state(HILDON_FILE_DETAILS_DIALOG(data),
-    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)));
-}
-
 static void
 hildon_file_details_dialog_map(GtkWidget *widget)
 {
@@ -249,6 +268,16 @@ hildon_file_details_dialog_map(GtkWidget *widget)
     gtk_widget_grab_focus(priv->ok_button);
 }
 
+static void
+permissions_insensitive_press (GtkWidget *widget, gpointer data)
+{
+  HildonFileDetailsDialog* self = HILDON_FILE_DETAILS_DIALOG (data);
+  const char *msg 
+    = gnome_vfs_result_to_string (self->priv->change_permissions_result);
+
+  hildon_banner_show_information (GTK_WIDGET (self), NULL, msg);
+}
+
 static void
 hildon_file_details_dialog_class_init(HildonFileDetailsDialogClass * klass)
 {
@@ -335,7 +364,6 @@ hildon_file_details_dialog_init(HildonFileDetailsDialog *self)
     GtkWidget *caption_read, *caption_device;
     GtkWidget *hbox_location, *hbox_device, *hbox_type;
     GtkWidget *vbox;
-    GtkWidget *scroll;
     GtkSizeGroup *group;
     GdkGeometry geometry;
 
@@ -347,7 +375,7 @@ hildon_file_details_dialog_init(HildonFileDetailsDialog *self)
                                 HildonFileDetailsDialogPrivate);
 
     priv->notebook = GTK_NOTEBOOK(gtk_notebook_new());
-    scroll = gtk_scrolled_window_new(NULL, NULL);
+    priv->scroll = gtk_scrolled_window_new(NULL, NULL);
     vbox = gtk_vbox_new(FALSE, 0);
     group = gtk_size_group_new(GTK_SIZE_GROUP_BOTH);
 
@@ -367,6 +395,8 @@ hildon_file_details_dialog_init(HildonFileDetailsDialog *self)
     priv->file_date = g_object_new(GTK_TYPE_LABEL,"xalign", 0.0f, NULL);
     priv->file_time = g_object_new(GTK_TYPE_LABEL, "xalign", 0.0f, NULL);
     priv->file_readonly = gtk_check_button_new();
+    g_signal_connect (priv->file_readonly, "insensitive-press",
+		      G_CALLBACK (permissions_insensitive_press), self);
 
     hbox_type = gtk_hbox_new (FALSE, HILDON_MARGIN_DEFAULT);
     hbox_location = gtk_hbox_new(FALSE, HILDON_MARGIN_DEFAULT);
@@ -435,23 +465,23 @@ hildon_file_details_dialog_init(HildonFileDetailsDialog *self)
     gtk_box_pack_start(GTK_BOX(vbox), caption_size, FALSE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(vbox), caption_read, FALSE, TRUE, 0);
 
-    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scroll), vbox);
-    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
-        GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-    gtk_container_set_border_width(GTK_CONTAINER(scroll),
+    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(priv->scroll), vbox);
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(priv->scroll),
+        GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+    gtk_container_set_border_width(GTK_CONTAINER(priv->scroll),
         HILDON_MARGIN_DEFAULT);
     /* Both scrolled window and viewport have separate shadows... */
-    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll),
+    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(priv->scroll),
         GTK_SHADOW_NONE);
     gtk_viewport_set_shadow_type(
-        GTK_VIEWPORT(gtk_bin_get_child(GTK_BIN(scroll))),
+        GTK_VIEWPORT(gtk_bin_get_child(GTK_BIN(priv->scroll))),
         GTK_SHADOW_NONE);
 
     gtk_container_set_focus_vadjustment(GTK_CONTAINER(vbox),
-      gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(scroll)));
+      gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(priv->scroll)));
 
     /* Populate dialog */
-    gtk_notebook_append_page(priv->notebook, scroll,
+    gtk_notebook_append_page(priv->notebook, priv->scroll,
                              gtk_label_new(_("sfil_ti_notebook_common")));
 
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(self)->vbox),
@@ -479,12 +509,10 @@ hildon_file_details_dialog_init(HildonFileDetailsDialog *self)
     gtk_dialog_add_button(GTK_DIALOG(self),
                           _("sfil_bd_filetype_details_dialog_cancel"),
                           GTK_RESPONSE_CANCEL);
-
-    priv->toggle_handler = g_signal_connect(priv->file_readonly, "toggled",
-      G_CALLBACK(hildon_file_details_dialog_read_only_toggled),
-      self);
     
-    gtk_widget_set_usize (GTK_WIDGET (self), 400, -1);
+    gtk_widget_set_size_request (GTK_WIDGET (self), 400, -1);
+	g_signal_connect (G_OBJECT (self), "show",
+			  G_CALLBACK (do_line_wrapping), priv);
 }
 
 static void
@@ -870,9 +898,13 @@ void hildon_file_details_dialog_set_file_iter(HildonFileDetailsDialog *self, Gtk
   g_signal_handler_block(self->priv->file_readonly, self->priv->toggle_handler);
 
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(self->priv->file_readonly),
-    location_readonly || !write_access(uri));
-  gtk_widget_set_sensitive(self->priv->file_readonly, !location_readonly);
+			       location_readonly || !write_access(uri));
 
+  self->priv->change_permissions_result =
+    guess_change_permissions_result (uri);
+  gtk_widget_set_sensitive(self->priv->file_readonly,
+			   (self->priv->change_permissions_result
+			    == GNOME_VFS_OK));
   self->priv->checkbox_original_state = gtk_toggle_button_get_active(
         GTK_TOGGLE_BUTTON(self->priv->file_readonly));
 
@@ -911,3 +943,78 @@ hildon_file_details_dialog_get_file_iter(HildonFileDetailsDialog *self, GtkTreeI
 
   return result;
 }
+
+/*
+	This function is a workaround to be able to show long labels on
+	HildonFileDetailsDialog without using horizontal scrolling.
+	This function is called after all the widgets are realized. so
+	that their actual size is known. Sizes are differnt in different
+	locales.
+*/
+static void
+do_line_wrapping (GtkWidget *w, gpointer data)
+{
+  HildonFileDetailsDialogPrivate *priv = (HildonFileDetailsDialogPrivate*)data;
+
+  GtkWidget *label;
+  GtkAllocation label_allocation;
+  GtkAllocation window_allocation;
+  
+  gint ideal_width;
+
+  label = priv->file_name;
+  label_allocation = label->allocation;
+  window_allocation = priv->scroll->allocation;
+
+  /* GtkWidgets are not aware of their parents size, so we have to explicitly
+     tell to the labels where their visibility ends --> wrapping point. 
+     Scrolled window is actually aware of it's size on screen so we adjust
+     the label widths according to it 
+  */
+  ideal_width = (window_allocation.width) - (label_allocation.x);
+
+  if (gtk_scrolled_window_get_vscrollbar (GTK_SCROLLED_WINDOW(priv->scroll)))
+    ideal_width -= 40; //Scroll bar width 
+
+  gtk_label_set_line_wrap (GTK_LABEL (priv->file_name), TRUE);
+  gtk_label_set_line_wrap_mode (GTK_LABEL (priv->file_name), PANGO_WRAP_CHAR);
+  
+  gtk_label_set_line_wrap (GTK_LABEL (priv->file_device), TRUE);
+  gtk_label_set_line_wrap_mode (GTK_LABEL (priv->file_device), PANGO_WRAP_CHAR);
+
+  gtk_label_set_line_wrap (GTK_LABEL (priv->file_location), TRUE);
+  gtk_label_set_line_wrap_mode (GTK_LABEL (priv->file_location), PANGO_WRAP_CHAR);
+
+  gtk_label_set_line_wrap (GTK_LABEL (priv->file_type), TRUE);
+  gtk_label_set_line_wrap_mode (GTK_LABEL (priv->file_type), PANGO_WRAP_CHAR);
+  
+  gtk_label_set_line_wrap (GTK_LABEL (priv->file_size), TRUE);
+  gtk_label_set_line_wrap_mode (GTK_LABEL (priv->file_size), PANGO_WRAP_CHAR);
+  
+  gtk_label_set_line_wrap (GTK_LABEL (priv->file_date), TRUE);
+  gtk_label_set_line_wrap_mode (GTK_LABEL (priv->file_date), PANGO_WRAP_CHAR);
+
+  gtk_label_set_line_wrap (GTK_LABEL (priv->file_time), TRUE);
+  gtk_label_set_line_wrap_mode (GTK_LABEL (priv->file_time), PANGO_WRAP_CHAR);
+
+  gtk_widget_set_size_request (GTK_WIDGET (priv->file_name), 
+			       ideal_width, -1);
+
+  gtk_widget_set_size_request (GTK_WIDGET (priv->file_device), 
+			       (ideal_width - 30), -1); //reduce icon width
+
+  gtk_widget_set_size_request (GTK_WIDGET (priv->file_location),
+			       (ideal_width - 30), -1); //reduce icon width
+
+  gtk_widget_set_size_request (GTK_WIDGET (priv->file_type),
+			       ideal_width, -1);
+
+  gtk_widget_set_size_request (GTK_WIDGET (priv->file_size), 
+			       ideal_width, -1);
+
+  gtk_widget_set_size_request (GTK_WIDGET (priv->file_date), 
+			       ideal_width, -1);
+
+  gtk_widget_set_size_request (GTK_WIDGET (priv->file_time), 
+			       ideal_width, -1);
+}
