@@ -43,11 +43,9 @@
 #include "hildon-file-common-private.h"
 #include "hildon-file-system-root.h"
 #include "hildon-file-system-local-device.h"
-#include "hildon-file-system-mmc.h"
 #include "hildon-file-system-upnp.h"
 #include "hildon-file-system-smb.h"
 #include "hildon-file-system-obex.h"
-#include "hildon-file-system-old-gateway.h"
 
 extern GtkFileSystem *gtk_file_system_unix_new();
 
@@ -76,9 +74,10 @@ _hildon_file_system_compare_ignore_last_separator(const char *a,
     len_a = strlen(a);
     len_b = strlen(b);
 
-    if (len_a > 1 && a[len_a - 1] == G_DIR_SEPARATOR)
+    while (len_a > 1 && a[len_a - 1] == G_DIR_SEPARATOR)
       len_a--;
-    if (len_b > 1 && b[len_b - 1] == G_DIR_SEPARATOR)
+
+    while (len_b > 1 && b[len_b - 1] == G_DIR_SEPARATOR)
       len_b--;
 
     if (len_a != len_b)
@@ -191,19 +190,13 @@ GNode *_hildon_file_system_get_locations(GtkFileSystem *fs)
             g_node_append_data(locations, location);
         }
 
-        if (!g_getenv("DISABLE_GATEWAY")) {
-            /* Setup gateway */
-            location = g_object_new(HILDON_TYPE_FILE_SYSTEM_OLD_GATEWAY, NULL);
-            g_node_append_data(locations, location);
-        } else {
-            /* Setup multiple Bluetooth device support */
-            env = g_getenv("HILDON_FM_OBEX_ROOT");
-            if (env && env[0]) {
-                location = g_object_new(HILDON_TYPE_FILE_SYSTEM_OBEX, NULL);
-                location->basepath = g_strdup(env);
-                g_node_append_data(locations, location);
-            }
-        }
+	/* Setup multiple Bluetooth device support */
+	env = g_getenv("HILDON_FM_OBEX_ROOT");
+	if (env && env[0]) {
+	  location = g_object_new(HILDON_TYPE_FILE_SYSTEM_OBEX, NULL);
+	  location->basepath = g_strdup(env);
+	  g_node_append_data(locations, location);
+	}
 
         /* Setup SMB
          */
@@ -303,7 +296,7 @@ _hildon_file_system_get_special_location(GtkFileSystem *fs,
         g_free(data.uri);
     }
 
-    if (data.result && data.is_child)
+    if (data.result)
       hildon_file_system_special_location_volumes_changed (data.result, fs);
 
     return data.result;
@@ -499,8 +492,8 @@ hex_digit_to_int (char d)
   return -1;
 }
 
-static char *
-unescape_string (const char *escaped)
+char *
+hildon_file_system_unescape_string (const char *escaped)
 {
   char *result = g_strdup (escaped);
   char *a, *b;
@@ -559,7 +552,7 @@ _hildon_file_system_create_file_name (GtkFileSystem *fs,
     name = g_strdup (gtk_file_info_get_display_name (info));
 
   if (name == NULL)
-    name = unescape_string (get_custom_root_name (path));
+    name = hildon_file_system_unescape_string (get_custom_root_name (path));
 
   return translate_special_name (name);
 }
